from uuid import uuid4
import os

from django.utils.deconstruct import deconstructible


@deconstructible
class ImageUploadTo(object):
    def __init__(self, sub_path='images/'):
        self.path = sub_path

    def __call__(self, instance, filename):
        ext = filename.split('.')[-1]
        filename = '{}/{}.{}'.format(uuid4().hex, uuid4().hex, ext)
        return os.path.join(self.path, filename)



from pydantic import BaseModel as Base
from functools import lru_cache

from django.conf import settings


class Gando(Base):
    MONITOR_KEYS: list = list()
    DEBUG: bool = True


@lru_cache()
def __get_settings():
    input_conf = settings.GANDO or {}
    return Gando(**input_conf)


SETTINGS = __get_settings()



from json import JSONEncoder
import datetime


class Encoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime.date, datetime.datetime)):
            return obj.isoformat()



from typing import Any
import json

from .encoders import Encoder


class JSON:

    @staticmethod
    def loads(value: str) -> Any:
        return json.loads(value)

    @staticmethod
    def dumps(value: Any) -> str:
        return json.dumps(value, cls=Encoder)



from pydantic import BaseModel
from typing import Optional
from . import MessagesSchema


class Response(BaseModel):
    success: bool = True
    has_warning: bool = False
    monitor: dict = dict()
    messages: Optional[MessagesSchema] = MessagesSchema()
    data: dict = dict()
    many: bool = False



from typing import List, Dict, Optional

from pydantic import BaseModel


class MessagesSchema(BaseModel):
    log_messages: Optional[List[Dict[str, str]]] = list()
    info_messages: Optional[List[Dict[str, str]]] = list()
    warning_messages: Optional[List[Dict[str, str]]] = list()
    error_messages: Optional[List[Dict[str, str]]] = list()
    exception_messages: Optional[List[Dict[str, str]]] = list()


class ResponseMessages:
    def __int__(self):
        self.__log = []
        self.__info = []
        self.__warning = []
        self.__error = []
        self.__exception = []

    @property
    def log(self):
        return self.__log

    @property
    def info(self):
        return self.__info

    @property
    def warning(self):
        return self.__warning

    @property
    def error(self):
        return self.__error

    @property
    def exception(self):
        return self.__exception

    def add2logs(self, msg_title: str, msg_content: str):
        self.__log.append({msg_title: msg_content})

    def add2infos(self, msg_title: str, msg_content: str):
        self.__info.append({msg_title: msg_content})

    def add2warnings(self, msg_title: str, msg_content: str):
        self.__warning.append({msg_title: msg_content})

    def add2errors(self, msg_title: str, msg_content: str):
        self.__error.append({msg_title: msg_content})

    def add2exceptions(self, msg_title: str, msg_content: str):
        self.__exception.append({msg_title: msg_content})

    def export(self):
        ret = MessagesSchema(
            log_messages=self.__log,
            info_messages=self.__info,
            warning_messages=self.__warning,
            error_messages=self.__error,
            exception_messages=self.__exception,
        )
        return ret



from typing import Any


class Monitor:
    def __int__(self):
        self.__monitor = {}

    def add(self, key: str, value: Any):
        self.__monitor[key] = value

    def delete(self, key):
        self.__monitor.pop(key, None)

    def export(self):
        return self.__monitor



from typing import Optional
import re

from django.utils.translation import gettext_lazy as _
from django.core.exceptions import ValidationError


def validate(value, pattern: Optional[str] = None, validation_error_message: Optional[str] = None):
    pattern = pattern or r'^[a-z0-9-]+$'
    validation_error_message = validation_error_message or _(
        'The value of this section should only contain lowercase letters, English numbers and - sign.')
    if not re.match(pattern, value):
        raise ValidationError(validation_error_message)



from django.core import validators
from django.utils.deconstruct import deconstructible
from django.utils.translation import gettext_lazy as _


@deconstructible
class UsernameValidator(validators.RegexValidator):
    regex = r"^(\+[0-9]{2,})$"
    message = _(
        "Enter a valid phone_number(example: +989123456789)."
    )
    flags = 0
